@@ -29,6 +29,7 @@
 #include <wtf/HashMap.h>
 #include <wtf/HashSet.h>
 #include <wtf/IteratorRange.h>
+#include <wtf/Variant.h>
 
 namespace JSC {
 
@@ -44,7 +45,10 @@ public:
     ALWAYS_INLINE bool isFunction() const { return m_bits & IsFunction; }
     ALWAYS_INLINE bool isParameter() const { return m_bits & IsParameter; }
     ALWAYS_INLINE bool isSloppyModeHoistingCandidate() const { return m_bits & IsSloppyModeHoistingCandidate; }
-    ALWAYS_INLINE bool isPrivateName() const { return m_bits & IsPrivateName; }
+    ALWAYS_INLINE bool isPrivateField() const { return m_bits & IsPrivateField; }
+    ALWAYS_INLINE bool isPrivateMethod() const { return m_bits & IsPrivateMethod; }
+    ALWAYS_INLINE bool isPrivateSetter() const { return m_bits & IsPrivateSetter; }
+    ALWAYS_INLINE bool isPrivateGetter() const { return m_bits & IsPrivateGetter; }
 
     ALWAYS_INLINE void setIsCaptured() { m_bits |= IsCaptured; }
     ALWAYS_INLINE void setIsConst() { m_bits |= IsConst; }
@@ -56,7 +60,10 @@ public:
     ALWAYS_INLINE void setIsFunction() { m_bits |= IsFunction; }
     ALWAYS_INLINE void setIsParameter() { m_bits |= IsParameter; }
     ALWAYS_INLINE void setIsSloppyModeHoistingCandidate() { m_bits |= IsSloppyModeHoistingCandidate; }
-    ALWAYS_INLINE void setIsPrivateName() { m_bits |= IsPrivateName; }
+    ALWAYS_INLINE void setIsPrivateField() { m_bits |= IsPrivateField; }
+    ALWAYS_INLINE void setIsPrivateMethod() { m_bits |= IsPrivateMethod; }
+    ALWAYS_INLINE void setIsPrivateSetter() { m_bits |= IsPrivateSetter; }
+    ALWAYS_INLINE void setIsPrivateGetter() { m_bits |= IsPrivateGetter; }
 
     ALWAYS_INLINE void clearIsVar() { m_bits &= ~IsVar; }
 
@@ -67,6 +74,8 @@ public:
         return m_bits == other.m_bits;
     }
 
+    void dump(PrintStream&) const;
+
 private:
     enum Traits : uint16_t {
         IsCaptured = 1 << 0,
@@ -79,7 +88,10 @@ private:
         IsFunction = 1 << 7,
         IsParameter = 1 << 8,
         IsSloppyModeHoistingCandidate = 1 << 9,
-        IsPrivateName = 1 << 10,
+        IsPrivateField = 1 << 10,
+        IsPrivateMethod = 1 << 11,
+        IsPrivateGetter = 1 << 12,
+        IsPrivateSetter = 1 << 13,
     };
     uint16_t m_bits { 0 };
 };
@@ -89,14 +101,21 @@ struct VariableEnvironmentEntryHashTraits : HashTraits<VariableEnvironmentEntry>
 };
 
 struct PrivateNameEntry {
+    friend class CachedPrivateNameEntry;
+
+    static constexpr unsigned privateClassBrandOffset = 0;
+    static constexpr unsigned privateBrandOffset = 1;
+
 public:
     PrivateNameEntry(uint16_t traits = 0) { m_bits = traits; }
 
-    ALWAYS_INLINE bool isUsed() const { return m_bits & IsUsed; }
-    ALWAYS_INLINE bool isDeclared() const { return m_bits & IsDeclared; }
+    ALWAYS_INLINE bool isMethod() const { return m_bits & IsMethod; }
+    ALWAYS_INLINE bool isSetter() const { return m_bits & IsSetter; }
+    ALWAYS_INLINE bool isGetter() const { return m_bits & IsGetter; }
+    ALWAYS_INLINE bool isField() const { return !isPrivateMethodOrAccessor(); }
+    ALWAYS_INLINE bool isStatic() const { return m_bits & IsStatic; }
 
-    ALWAYS_INLINE void setIsUsed() { m_bits |= IsUsed; }
-    ALWAYS_INLINE void setIsDeclared() { m_bits |= IsDeclared; }
+    bool isPrivateMethodOrAccessor() const { return isMethod() || isSetter() || isGetter(); }
 
     uint16_t bits() const { return m_bits; }
 
@@ -106,8 +125,11 @@ public:
     }
 
     enum Traits : uint16_t {
-        IsUsed = 1 << 0,
-        IsDeclared = 1 << 1,
+        None = 0,
+        IsMethod = 1 << 0,
+        IsGetter = 1 << 1,
+        IsSetter = 1 << 2,
+        IsStatic = 1 << 3,
     };
 
 private:
@@ -115,14 +137,18 @@ private:
 };
 
 struct PrivateNameEntryHashTraits : HashTraits<PrivateNameEntry> {
-    static const bool needsDestruction = false;
+    static constexpr bool needsDestruction = false;
 };
 
+typedef HashMap<PackedRefPtr<UniquedStringImpl>, PrivateNameEntry, IdentifierRepHash, HashTraits<RefPtr<UniquedStringImpl>>, PrivateNameEntryHashTraits> PrivateNameEnvironment;
+
 class VariableEnvironment {
+    WTF_MAKE_FAST_ALLOCATED;
 private:
     typedef HashMap<PackedRefPtr<UniquedStringImpl>, VariableEnvironmentEntry, IdentifierRepHash, HashTraits<RefPtr<UniquedStringImpl>>, VariableEnvironmentEntryHashTraits> Map;
-    typedef HashMap<PackedRefPtr<UniquedStringImpl>, PrivateNameEntry, IdentifierRepHash, HashTraits<RefPtr<UniquedStringImpl>>, PrivateNameEntryHashTraits> PrivateNames;
+
 public:
+
     VariableEnvironment() { }
     VariableEnvironment(VariableEnvironment&& other)
         : m_map(WTFMove(other.m_map))
@@ -144,6 +170,16 @@ public:
     ALWAYS_INLINE Map::const_iterator end() const { return m_map.end(); }
     ALWAYS_INLINE Map::AddResult add(const RefPtr<UniquedStringImpl>& identifier) { return m_map.add(identifier, VariableEnvironmentEntry()); }
     ALWAYS_INLINE Map::AddResult add(const Identifier& identifier) { return add(identifier.impl()); }
+
+    ALWAYS_INLINE PrivateNameEnvironment::AddResult addPrivateName(const Identifier& identifier) { return addPrivateName(identifier.impl()); }
+    ALWAYS_INLINE PrivateNameEnvironment::AddResult addPrivateName(const RefPtr<UniquedStringImpl>& identifier)
+    {
+        if (!m_rareData)
+            m_rareData = makeUnique<VariableEnvironment::RareData>();
+
+        return m_rareData->m_privateNames.add(identifier, PrivateNameEntry());
+    }
+
     ALWAYS_INLINE unsigned size() const { return m_map.size() + privateNamesSize(); }
     ALWAYS_INLINE unsigned mapSize() const { return m_map.size(); }
     ALWAYS_INLINE bool contains(const RefPtr<UniquedStringImpl>& identifier) const { return m_map.contains(identifier); }
@@ -160,31 +196,38 @@ public:
     void markVariableAsExported(const RefPtr<UniquedStringImpl>& identifier);
 
     bool isEverythingCaptured() const { return m_isEverythingCaptured; }
-    bool isEmpty() const { return !m_map.size(); }
+    bool isEmpty() const { return !m_map.size() && !privateNamesSize(); }
 
-    using PrivateNamesRange = WTF::IteratorRange<PrivateNames::iterator>;
+    using PrivateNamesRange = WTF::IteratorRange<PrivateNameEnvironment::iterator>;
 
-    ALWAYS_INLINE Map::AddResult declarePrivateName(const Identifier& identifier) { return declarePrivateName(identifier.impl()); }
-    ALWAYS_INLINE void usePrivateName(const Identifier& identifier) { usePrivateName(identifier.impl()); }
+    ALWAYS_INLINE Map::AddResult declarePrivateField(const Identifier& identifier) { return declarePrivateField(identifier.impl()); }
 
-    Map::AddResult declarePrivateName(const RefPtr<UniquedStringImpl>& identifier)
+    bool declarePrivateMethod(const Identifier& identifier) { return declarePrivateMethod(identifier.impl()); }
+    bool declarePrivateMethod(const RefPtr<UniquedStringImpl>& identifier, PrivateNameEntry::Traits addionalTraits = PrivateNameEntry::Traits::None);
+
+    enum class PrivateDeclarationResult {
+        Success,
+        DuplicatedName,
+        InvalidStaticNonStatic
+    };
+
+    PrivateDeclarationResult declarePrivateAccessor(const RefPtr<UniquedStringImpl>&, PrivateNameEntry accessorTraits);
+    
+    bool declareStaticPrivateMethod(const Identifier& identifier)
     {
-        auto& meta = getOrAddPrivateName(identifier.get());
-        meta.setIsDeclared();
-        auto entry = VariableEnvironmentEntry();
-        entry.setIsPrivateName();
-        entry.setIsConst();
-        entry.setIsCaptured();
-        return m_map.add(identifier, entry);
-    }
-    void usePrivateName(const RefPtr<UniquedStringImpl>& identifier)
-    {
-        auto& meta = getOrAddPrivateName(identifier.get());
-        meta.setIsUsed();
-        if (meta.isDeclared())
-            find(identifier)->value.setIsCaptured();
+        return declarePrivateMethod(identifier.impl(), static_cast<PrivateNameEntry::Traits>(PrivateNameEntry::Traits::IsMethod | PrivateNameEntry::Traits::IsStatic));
     }
 
+    PrivateDeclarationResult declarePrivateSetter(const Identifier& identifier) { return declarePrivateSetter(identifier.impl()); }
+    PrivateDeclarationResult declareStaticPrivateSetter(const Identifier& identifier) { return declarePrivateSetter(identifier.impl(), PrivateNameEntry::Traits::IsStatic); }
+    PrivateDeclarationResult declarePrivateSetter(const RefPtr<UniquedStringImpl>& identifier, PrivateNameEntry::Traits modifierTraits = PrivateNameEntry::Traits::None);
+
+    PrivateDeclarationResult declarePrivateGetter(const Identifier& identifier) { return declarePrivateGetter(identifier.impl()); }
+    PrivateDeclarationResult declareStaticPrivateGetter(const Identifier& identifier) { return declarePrivateGetter(identifier.impl(), PrivateNameEntry::Traits::IsStatic); }
+    PrivateDeclarationResult declarePrivateGetter(const RefPtr<UniquedStringImpl>& identifier, PrivateNameEntry::Traits modifierTraits = PrivateNameEntry::Traits::None);
+
+    Map::AddResult declarePrivateField(const RefPtr<UniquedStringImpl>&);
+
     ALWAYS_INLINE PrivateNamesRange privateNames() const
     {
         // Use of the IteratorRange must be guarded to prevent ASSERT failures in checkValidity().
@@ -199,6 +242,46 @@ public:
         return m_rareData->m_privateNames.size();
     }
 
+    ALWAYS_INLINE PrivateNameEnvironment* privateNameEnvironment()
+    {
+        if (!m_rareData)
+            return nullptr;
+        return &m_rareData->m_privateNames;
+    }
+
+    ALWAYS_INLINE const PrivateNameEnvironment* privateNameEnvironment() const
+    {
+        if (!m_rareData)
+            return nullptr;
+        return &m_rareData->m_privateNames;
+    }
+
+    ALWAYS_INLINE bool hasStaticPrivateMethodOrAccessor() const
+    {
+        if (!m_rareData)
+            return false;
+
+        for (auto entry : privateNames()) {
+            if (entry.value.isPrivateMethodOrAccessor() && entry.value.isStatic())
+                return true;
+        }
+        
+        return false;
+    }
+    
+    ALWAYS_INLINE bool hasInstancePrivateMethodOrAccessor() const
+    {
+        if (!m_rareData)
+            return false;
+        
+        for (auto entry : privateNames()) {
+            if (entry.value.isPrivateMethodOrAccessor() && !entry.value.isStatic())
+                return true;
+        }
+        
+        return false;
+    }
+
     ALWAYS_INLINE bool hasPrivateName(const Identifier& identifier)
     {
         if (!m_rareData)
@@ -206,30 +289,16 @@ public:
         return m_rareData->m_privateNames.contains(identifier.impl());
     }
 
-    ALWAYS_INLINE void copyPrivateNamesTo(VariableEnvironment& other) const
+    ALWAYS_INLINE void addPrivateNamesFrom(const PrivateNameEnvironment* privateNameEnvironment)
     {
-        if (!m_rareData)
+        if (!privateNameEnvironment)
             return;
-        if (!other.m_rareData)
-            other.m_rareData = WTF::makeUnique<VariableEnvironment::RareData>();
-        if (privateNamesSize() > 0) {
-            for (auto entry : privateNames()) {
-                if (!(entry.value.isUsed() && entry.value.isDeclared()))
-                    other.m_rareData->m_privateNames.add(entry.key, entry.value);
-            }
-        }
-    }
 
-    ALWAYS_INLINE void copyUndeclaredPrivateNamesTo(VariableEnvironment& outer) const {
-        // Used by the Parser to transfer recorded uses of PrivateNames from an
-        // inner PrivateNameEnvironment into an outer one, in case a PNE is used
-        // earlier in the source code than it is defined.
-        if (privateNamesSize() > 0) {
-            for (auto entry : privateNames()) {
-                if (entry.value.isUsed() && !entry.value.isDeclared())
-                    outer.getOrAddPrivateName(entry.key.get()).setIsUsed();
-            }
-        }
+        if (!m_rareData)
+            m_rareData = makeUnique<VariableEnvironment::RareData>();
+
+        for (auto entry : *privateNameEnvironment)
+            m_rareData->m_privateNames.add(entry.key, entry.value);
     }
 
     struct RareData {
@@ -242,9 +311,11 @@ public:
         }
         RareData(const RareData&) = default;
         RareData& operator=(const RareData&) = default;
-        PrivateNames m_privateNames;
+        PrivateNameEnvironment m_privateNames;
     };
 
+    void dump(PrintStream&) const;
+
 private:
     friend class CachedVariableEnvironment;
 
@@ -262,56 +333,69 @@ private:
     std::unique_ptr<VariableEnvironment::RareData> m_rareData;
 };
 
-class CompactVariableEnvironment {
-    WTF_MAKE_FAST_ALLOCATED;
-    WTF_MAKE_NONCOPYABLE(CompactVariableEnvironment);
+using TDZEnvironment = HashSet<RefPtr<UniquedStringImpl>, IdentifierRepHash>;
 
-    friend class CachedCompactVariableEnvironment;
+class CompactTDZEnvironment {
+    WTF_MAKE_FAST_ALLOCATED;
+    WTF_MAKE_NONCOPYABLE(CompactTDZEnvironment);
+
+    friend class CachedCompactTDZEnvironment;
+
+    using Compact = Vector<PackedRefPtr<UniquedStringImpl>>;
+    using Inflated = TDZEnvironment;
+    using Variables = Variant<Compact, Inflated>;
 
 public:
-    CompactVariableEnvironment(const VariableEnvironment&);
-    VariableEnvironment toVariableEnvironment() const;
+    CompactTDZEnvironment(const TDZEnvironment&);
 
-    bool operator==(const CompactVariableEnvironment&) const;
+    bool operator==(const CompactTDZEnvironment&) const;
     unsigned hash() const { return m_hash; }
 
-private:
-    CompactVariableEnvironment() = default;
+    static void sortCompact(Compact&);
 
-    Vector<PackedRefPtr<UniquedStringImpl>> m_variables;
-    Vector<VariableEnvironmentEntry> m_variableMetadata;
+    TDZEnvironment& toTDZEnvironment() const
+    {
+        if (WTF::holds_alternative<Inflated>(m_variables))
+            return const_cast<TDZEnvironment&>(WTF::get<Inflated>(m_variables));
+        return toTDZEnvironmentSlow();
+    }
+
+private:
+    CompactTDZEnvironment() = default;
+    TDZEnvironment& toTDZEnvironmentSlow() const;
+
+    mutable Variables m_variables;
     unsigned m_hash;
-    bool m_isEverythingCaptured;
 };
 
-struct CompactVariableMapKey {
-    CompactVariableMapKey()
+struct CompactTDZEnvironmentKey {
+    CompactTDZEnvironmentKey()
         : m_environment(nullptr)
     {
         ASSERT(isHashTableEmptyValue());
     }
 
-    CompactVariableMapKey(CompactVariableEnvironment& environment)
+    CompactTDZEnvironmentKey(CompactTDZEnvironment& environment)
         : m_environment(&environment)
     { }
 
-    static unsigned hash(const CompactVariableMapKey& key) { return key.m_environment->hash(); }
-    static bool equal(const CompactVariableMapKey& a, const CompactVariableMapKey& b) { return *a.m_environment == *b.m_environment; }
+    static unsigned hash(const CompactTDZEnvironmentKey& key) { return key.m_environment->hash(); }
+    static bool equal(const CompactTDZEnvironmentKey& a, const CompactTDZEnvironmentKey& b) { return *a.m_environment == *b.m_environment; }
     static constexpr bool safeToCompareToEmptyOrDeleted = false;
-    static void makeDeletedValue(CompactVariableMapKey& key)
+    static void makeDeletedValue(CompactTDZEnvironmentKey& key)
     {
-        key.m_environment = reinterpret_cast<CompactVariableEnvironment*>(1);
+        key.m_environment = reinterpret_cast<CompactTDZEnvironment*>(1);
     }
     bool isHashTableDeletedValue() const
     {
-        return m_environment == reinterpret_cast<CompactVariableEnvironment*>(1);
+        return m_environment == reinterpret_cast<CompactTDZEnvironment*>(1);
     }
     bool isHashTableEmptyValue() const
     {
         return !m_environment;
     }
 
-    CompactVariableEnvironment& environment()
+    CompactTDZEnvironment& environment()
     {
         RELEASE_ASSERT(!isHashTableDeletedValue());
         RELEASE_ASSERT(!isHashTableEmptyValue());
@@ -319,7 +403,7 @@ struct CompactVariableMapKey {
     }
 
 private:
-    CompactVariableEnvironment* m_environment;
+    CompactTDZEnvironment* m_environment;
 };
 
 } // namespace JSC
@@ -327,34 +411,32 @@ private:
 namespace WTF {
 
 template<typename T> struct DefaultHash;
-template<> struct DefaultHash<JSC::CompactVariableMapKey> {
-    using Hash = JSC::CompactVariableMapKey;
-};
+template<> struct DefaultHash<JSC::CompactTDZEnvironmentKey> : JSC::CompactTDZEnvironmentKey { };
 
-template<> struct HashTraits<JSC::CompactVariableMapKey> : GenericHashTraits<JSC::CompactVariableMapKey> {
+template<> struct HashTraits<JSC::CompactTDZEnvironmentKey> : GenericHashTraits<JSC::CompactTDZEnvironmentKey> {
     static constexpr bool emptyValueIsZero = true;
-    static JSC::CompactVariableMapKey emptyValue() { return JSC::CompactVariableMapKey(); }
+    static JSC::CompactTDZEnvironmentKey emptyValue() { return JSC::CompactTDZEnvironmentKey(); }
 
     static constexpr bool hasIsEmptyValueFunction = true;
-    static bool isEmptyValue(JSC::CompactVariableMapKey key) { return key.isHashTableEmptyValue(); }
+    static bool isEmptyValue(JSC::CompactTDZEnvironmentKey key) { return key.isHashTableEmptyValue(); }
 
-    static void constructDeletedValue(JSC::CompactVariableMapKey& key) { JSC::CompactVariableMapKey::makeDeletedValue(key); }
-    static bool isDeletedValue(JSC::CompactVariableMapKey key) { return key.isHashTableDeletedValue(); }
+    static void constructDeletedValue(JSC::CompactTDZEnvironmentKey& key) { JSC::CompactTDZEnvironmentKey::makeDeletedValue(key); }
+    static bool isDeletedValue(JSC::CompactTDZEnvironmentKey key) { return key.isHashTableDeletedValue(); }
 };
 
 } // namespace WTF
 
 namespace JSC {
 
-class CompactVariableMap : public RefCounted<CompactVariableMap> {
+class CompactTDZEnvironmentMap : public RefCounted<CompactTDZEnvironmentMap> {
 public:
     class Handle {
-        friend class CachedCompactVariableMapHandle;
+        friend class CachedCompactTDZEnvironmentMapHandle;
 
     public:
         Handle() = default;
 
-        Handle(CompactVariableEnvironment&, CompactVariableMap&);
+        Handle(CompactTDZEnvironment&, CompactTDZEnvironmentMap&);
 
         Handle(Handle&& other)
         {
@@ -379,7 +461,7 @@ public:
 
         explicit operator bool() const { return !!m_map; }
 
-        const CompactVariableEnvironment& environment() const
+        const CompactTDZEnvironment& environment() const
         {
             return *m_environment;
         }
@@ -391,19 +473,41 @@ public:
             std::swap(other.m_map, m_map);
         }
 
-        CompactVariableEnvironment* m_environment { nullptr };
-        RefPtr<CompactVariableMap> m_map;
+        CompactTDZEnvironment* m_environment { nullptr };
+        RefPtr<CompactTDZEnvironmentMap> m_map;
     };
 
-    Handle get(const VariableEnvironment&);
+    Handle get(const TDZEnvironment&);
 
 private:
     friend class Handle;
-    friend class CachedCompactVariableMapHandle;
+    friend class CachedCompactTDZEnvironmentMapHandle;
 
-    Handle get(CompactVariableEnvironment*, bool& isNewEntry);
+    Handle get(CompactTDZEnvironment*, bool& isNewEntry);
 
-    HashMap<CompactVariableMapKey, unsigned> m_map;
+    HashMap<CompactTDZEnvironmentKey, unsigned> m_map;
+};
+
+class TDZEnvironmentLink : public RefCounted<TDZEnvironmentLink> {
+    TDZEnvironmentLink(CompactTDZEnvironmentMap::Handle handle, RefPtr<TDZEnvironmentLink> parent)
+        : m_handle(WTFMove(handle))
+        , m_parent(WTFMove(parent))
+    { }
+
+public:
+    static RefPtr<TDZEnvironmentLink> create(CompactTDZEnvironmentMap::Handle handle, RefPtr<TDZEnvironmentLink> parent)
+    {
+        return adoptRef(new TDZEnvironmentLink(WTFMove(handle), WTFMove(parent)));
+    }
+
+    bool contains(UniquedStringImpl* impl) const { return m_handle.environment().toTDZEnvironment().contains(impl); }
+    TDZEnvironmentLink* parent() { return m_parent.get(); }
+
+private:
+    friend class CachedTDZEnvironmentLink;
+
+    CompactTDZEnvironmentMap::Handle m_handle;
+    RefPtr<TDZEnvironmentLink> m_parent;
 };
 
 } // namespace JSC
