@@ -27,6 +27,7 @@
 
 #include "APIObject.h"
 #include "ContentWorldShared.h"
+#include "DataReference.h"
 #include "MessageReceiver.h"
 #include "UserContentControllerIdentifier.h"
 #include "WebPageProxyIdentifier.h"
@@ -48,10 +49,6 @@ class UserScript;
 class UserStyleSheet;
 }
 
-namespace IPC {
-class DataReference;
-}
-
 namespace WebCore {
 struct SecurityOriginData;
 }
